<?php

namespace Drupal\cforge\EventSubscriber;

use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\Routing\Exception\RouteNotFoundException;

/**
 * Cforge react to events.
 *
 * @todo inject Drupal::Config('cforge.settings') and current_route.match, and url_generator
 */
class CforgeSubscriber implements EventSubscriberInterface {

  private $currentUser;
  private $cforgeSettings;
  private $routeMatch;

  function __construct($current_user, $config, $route_match) {
    $this->currentUser = $current_user;
    $this->cforgeSettings = $config->get('cforge.settings');
    $this->routeMatch = $route_match;
  }

  /**
   * Redirect users
   */
  public function cforgeRedirect(GetResponseEvent $event) {
    if ($this->currentUser->isAuthenticated()) {
      if ($this->cforgeSettings->get('flavour')) {
        if ($this->currentUser->id() == 1) {
          $route_name = $this->routeMatch->getRouteName();
          //these are all routes which can be invoked BEFORE cforge.setup
          $dont_divert = [
            'cforge.setup',
            'system.batch_page.json',
            'system.batch_page.html',
            'user.page',
            'entity.node.canonical' // Any node page, including the front page
          ];
          if (!in_array($route_name, $dont_divert)) {
            drupal_set_message('diverting from <em>'.$route_name .'</em> with flavour: '.$this->cforgeSettings->get('flavour'));
            // Redirect to the setup wizard
            $response = new RedirectResponse(Url::fromRoute('cforge.setup')->toString());
            $event->setResponse($response);
          }
        }
      }
      // Different front page for anon and authenticated users.
      elseif ($event->getRequest()->getpathinfo() == '/') {
        $response = new RedirectResponse($this->cforgeSettings->get('member_frontpage'));
        $event->setResponse($response);
      }
    }
    else {
      $path = trim($event->getRequest()->getpathinfo(), '/');
      // If there was a trailing slash filter the empty part.
      $parts = array_filter(explode('/', $path));
      if (count($parts) > 5) {
        throw new RouteNotFoundException('Access denied');
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['cforgeRedirect'];
    return $events;
  }

}
