<?php

namespace Drupal\cforge\Plugin\Block;

use Drupal\user\Entity\User;
use Drupal\user\Entity\Role;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Shows usernames and passwords for first accounts
 *
 * @Block(
 *   id = "demo_logins",
 *   admin_label = @Translation("Demo login details"),
 *   category = @Translation("Hamlets")
 * )
 */
class DemoLogins extends BlockBase implements ContainerFactoryPluginInterface {

  private $cforgeSettings;

  private $entityQuery;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, $config_factory, $entity_query) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->cforgeSettings = $config_factory->get('cforge.settings');
    $this->entityQuery = $entity_query;
  }


  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('entity.query')
    );
  }

  /**
   * {@inheritdoc}
   *
   * @todo Only access the block when we are in demo mode
   */
  protected function blockAccess(AccountInterface $account) {
    $result = $this->cforgeSettings->get('demo_mode') //&& $account->isAnonymous()
      ? AccessResult::allowed()
      : AccessResult::forbidden();
    return $result->addCacheableDependency($this->cforgeSettings);
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $markup[] = [
      '#markup' => $this->t('Login with any of following usernames:'),
    ];
    $markup[] = [
      '#theme' => 'item_list',
      '#items' => $this->getNames(),
      // Need some css to remove the bullet points
      '#type' => 'ul',
    ];
    $markup[] = [
      '#markup' => $this->t("and password 'demo'")
    ];
    return $markup;
  }

  public function getNames() {
    $names = [];
    foreach (Role::loadMultiple() as $rid => $role) {
      if (in_array($rid, ['anonymous', 'authenticated'])) {
        continue;
      }
      $uids = $this->entityQuery->get('user')->condition('roles', $rid)->condition('uid', 1, '<>')->execute();
      if (empty($uids)) {
        continue;
      }
      $names[] = User::load(reset($uids))->getUsername() . '('.$role->label().')';
    }
    return $names;
  }

}
